#!/config/sh                              # just and only for syntax highlighting
#********************************************************************************
# qware build environment - header file of the qube configure script            *
#                                                                               *
#  Author: Kai Peter, ©2020-present                                             *
# Version: 0.8.1                                                                *
# License: ISC                                                                  *
#                                                                               *
# Description: Standard commands and functions of the qube configure process.   *
#              Only developers should adapt it to their package requirements.   *
#********************************************************************************
declare -r QUBEDIR=$1       # required (defined in the 'configure' script itself)
[ $DBG ] ||  DBG=0          # initialize debug level (default: off)
#----- default functions (basic) ------------------------------------------------
# general configure error handling: print message given by "$1" and exit
error() { [ "$1" ] && printf "\033[91mError: %s\033[0m\n" "$1"; exit 1; }
log() { printf "$1" | tee -a config.log; }

getSystype() {
  log "Checking system ... "
  OS=`uname -s | tr '[A-Z]' '[a-z]'`
  [ -z "$OS" ] && log "failed!\n" && error "Unable to detect system type!"
  log "detected $OS\n"
  log "Checking architecture ... "
  local ARCH=`uname -m`
  echo "$OS:$ARCH" > systype
  log "build on $ARCH!\n"
}

mkShebang() {
  QUBE_SHEBANG=`which env` || exit 1   # an error shouldn't happen, but who knows
  echo "#!$QUBE_SHEBANG bash" > ac-warn.sh   # output was formerly 'warn-auto.sh'
  echo "# !!! WARNING: This file was auto-generated. Do NOT edit!" >> ac-warn.sh
  # Hint: To use a different interpreter than 'sh' you can do something like this
  #       in a Makefile recipe (just an example to get the idea):
  #
  #       cat ac-warn.sh | sed 's/ sh$/ perl/g' | ...

  return
}

mkNIL() {
#  local DEV=`df | grep /dev$ | awk '{print $NF;}'`
  local DEV=`df | awk '/\/dev$/{print $NF;}'`
  [ -c $DEV/null ] && NIL=$DEV/null && return || \
    error "Cannot find $DEV/null - aborting!"
}
#**** begin processing **********************************************************
mkNIL; echo "$NIL" > ac-devnull                   # important: do this very early
shift                 # remove $QUBEDIR from arguments list ($1 from 'configure')
echo "Command line: configure $@" >> config.log
make clean &>$NIL                                 # clean up silently

# handle the control directory
[ -f $QUBEDIR/control/no-binpath   ] && NOBINPATH=1
[ -f $QUBEDIR/control/no-compile   ] && NOCOMPILE=1
[ -f $QUBEDIR/control/no-externals ] && NOGETDEPS=1     # get-externals
[ -f $QUBEDIR/control/no-layout    ] && NOLAYOUT=1
[ -f $QUBEDIR/control/no-libpath   ] && NOLIBPATH=1
[ -f $QUBEDIR/control/no-loadbin   ] && NOLOADBIN=1
[ -f $QUBEDIR/control/no-makelib   ] && NOMAKELIB=1
[ -f $QUBEDIR/control/no-pathes    ] && NOPATHES=1      # search pathes
[ -f $QUBEDIR/control/no-prefix    ] && NOPREFIX=1      # deprecated (senseless)
[ -f $QUBEDIR/control/no-symlinks  ] && NOSYMLINKS=1
[ -f $QUBEDIR/control/no-sysdeps   ] && NOSETDEPS=1     # set-sysdeps
[ -f $QUBEDIR/control/no-usrgrp    ] && NOUSRGRP=1

# get variables from 'conf-build' and corresponding 'conf-*' files **************
# conf-build is required, must be present in any case
[ -f "$QUBEDIR/conf-build" ] && source "$QUBEDIR/conf-build" 2>$NIL || \
  error "Unable to read '$QUBEDIR/conf-build'"
[ -f conf-build  ] && source ./conf-build            # deprecated
[ -f conf-prefix ] && PREFIX=`head -1 conf-prefix`   # deprecated, use conf-local
[ -f conf-suffix ] && SUFFIX=`head -1 conf-suffix`   # deprecated, use conf-local
# there could be either the one or the other file (user or distribution specific)
[ -f conf-local  ] && source ./conf-local || \
[ -f conf-dist   ] && source ./conf-dist

[ "$@" ] && source $QUBEDIR/configure.opts 2>$NIL
[ `type -t cmdArgs` ] && cmdArgs $@               # handle command line parameter

# overwrite some settings with command line arguments (if any)
[ "$argPREFIX" ] && PREFIX="$argPREFIX"
[ "$argSUFFIX" ] && SUFFIX="$argSUFFIX"

# include (source) the files containing the qube standard functions ...
F="$QUBEDIR/get-layout"    ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/get-externals" ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/get-usrcreds"  ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/mk-buildtools" ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/set-sysdeps"   ; [ -f "$F" ] && source "$F"
unset F
# ... and execute the qube standard functions
[ ! "$SYSTYPE" ] && getSystype
[ `type -t getLayout` ]    && getLayout
[ `type -t chkExtRes` ]    && chkExtRes
[ `type -t getUsrCreds` ]  && getUsrCreds
[ `type -t mkBuildTools` ] && mkBuildTools
[ `type -t setSysDeps` ]   && setSysDeps
. "$QUBEDIR"/mk-autotpl || error "Missing file mk-autotpl!"

# The last steps: create 'ac-warn.sh' and 'ac-devnull' always *******************
# Usually these files are required in 99.9% of build cases anyway, otherwise they
# could be used in the Makefile to check if the package was configured already.
[ -f ac-warn.sh ] || mkShebang
echo "$NIL" > ac-devnull

[ -f "$QUBEDIR/package.inc" ] && source "$QUBEDIR/package.inc"

echo "Configure finished successful." >> config.log
exit $?
